#include "mp_simulation.h"


int main(int argc, char **argv){
  if (argc < 3) {
    cerr << "No input file" << endl;
    return (-1);
  }

  // deal with options
  int flags = 0;
  int iter = -1;
  for (int i=3; i<argc; i++){
    if (strcmp(argv[i], "-f") == 0)      {flags = flags & (~BWD);}
    else if (strcmp(argv[i], "-b") == 0) {flags = flags | BWD;}
    else if (strcmp(argv[i], "-debug") == 0) {flags = flags | DEBUGPTM;}
    else if (strcmp(argv[i], "-output") == 0) {flags = flags | OUTPUT;}
    else if (strcmp(argv[i], "-outputfile") == 0) {flags = flags | OUTPUTFILE;}
    else if (strcmp(argv[i], "-iter") == 0 && argc >= i+1) {iter = atoi(argv[i+1]);}
    else cerr << "Unknown option :" << argv[i] << endl;
  }

  Aton<string> xnodes;     // Nodes of X
  Aton<string> ynodes;     // Nodes of Y
  Aton<string> ac;         // Ac
  int n, m, a;             // Size of X, Y, AP

  ifstream ifs1, ifs2;
  ifs1.open(argv[1]);
  if (!ifs1){
    cerr << "Error: cannot open file " << argv[1] <<endl;
    return (-1);
  }
  ifs2.open(argv[2]);
  if (!ifs2){
    cerr << "Error: cannot open file " << argv[2] <<endl;
    return (-1);
  }
  
  read_pre(ifs1, xnodes, ac);
  read_pre(ifs2, ynodes, ac);
  n = xnodes.size();
  m = ynodes.size();
  a = ac.size();

  map<int,double> **c, **d;
  map<int,double> cv, dv, ci, di;

  c  = new map<int,double>* [a];
  d  = new map<int,double>* [a];
  for (int i=0; i<a; i++){
    c[i] = new map<int,double> [n];
    d[i] = new map<int,double> [m];
  }

  read_body(ifs1, c, ci, cv, xnodes, ac, !(flags & BWD));
  read_body(ifs2, d, di, dv, ynodes, ac, flags & BWD);
  ifs1.close();
  ifs2.close();
  
  // if n,m,or a is 0, exit
  if (n<=0) {cerr << "n=0" << endl; return(-1);}
  if (m<=0) {cerr << "m=0" << endl; return(-1);}
  if (a<=0) {cerr << "a=0" << endl; return(-1);}

  // make output file name
  string fn, fm;
  fn = argv[1];
  fm = argv[2];
  string::size_type pos;

  if ((pos = fn.find_last_of("/")) != string::npos) fn = fn.substr(pos+1);
  if ((pos = fn.find_first_of(".")) != string::npos) fn = fn.substr(0,pos);
  if ((pos = fm.find_last_of("/")) != string::npos) fm = fm.substr(pos+1);
  if ((pos = fm.find_first_of(".")) != string::npos) fm = fm.substr(0,pos);
  fn = fn + fm;

  if (flags & DEBUGPTM){
    string s;
    output_input("c",c, "ci",ci, "cv",cv, a, n, !(flags & BWD));
    output_input("d",d, "di",di, "dv",dv, a, m, flags & BWD);
  }

  ofstream ofs;
  cout << "States : " << n << ", " << m << " Transitions : " << a << endl;
  
  // solve tropical linear inequality
  double *x;
  x = new double[n*m];
  int r = 0;
    string simtype;
    if (!(flags & BWD)) {
      simtype = "Forward Simulation";
      fm = fn + "_f.out";
      cout << "Searching for " << simtype << " ... " << endl;
      r = solve(a,n,m,c,ci,cv,d,di,dv,x,flags,iter);
    }
    else{
      simtype = "Backward Simulation";
      fm = fn + "_b.out";
      cout << "Searching for " << simtype << " ... " << endl;
      r = solve(a,m,n,d,di,dv,c,ci,cv,x,flags,iter);
    }

  switch (r){
      case SIMULATION_FOUND:
      if (flags & OUTPUT){
        for (int i=0; i<n; i++){
          for (int j=0; j<m; j++) {
            if (x[i*n+j] == -numeric_limits<double>::infinity()) cerr << setw(11) << "-inf" << " ";
            else cerr << setw(11) << x[i*n+j] << " ";
          }
          cerr << endl;
        }
      }
      if(flags & OUTPUTFILE){
        ofs.open(fm.c_str());
        if (!ofs) cerr << "Could not open " << fm << endl;
        for (int i=0; i<n; i++){
          for (int j=0; j<m; j++) {
            if (x[i*n+j] == -numeric_limits<double>::infinity()) ofs << setw(11) << "-inf" << " ";
            else ofs << setw(11) << x[i*n+j] << " ";
          }
          ofs << endl;
        }
        ofs.close();
      }
      cout << simtype << " Exists" << endl;
      break;
      
      case SIMULATION_NOTFOUND:
      cout << simtype << " does NOT exist" << endl;
      break;
      
      case ITERATION_LIMIT:
      cout << "Iteration limit achieved" << endl;
  }

  cout << endl;
  delete [] x;

  // free memory
  for(int i=0; i<a; i++){
    delete [] c[i];
    delete [] d[i];
  }
  delete [] c;
  delete [] d;

  return (0);
}
